<?php
/**
 * @package    Mailto2CE
 * @author     Douglas Machado {@link http://ideal.fok.com.br}
 * @author     Created on 08-Nov-2010
 * @license		GNU/GPL, see license.txt
 * Mailto2CE is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses. 
 */

//-- No direct access
defined('_JEXEC') or die('Access Denied - Please do not try to fool me! ;-)');

jimport('joomla.plugin.plugin');

/**
 * @package		mailto2ce
 * @subpackage	Contact Enhanced
 */
class plgSystemmailto2ce extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @access      protected
	 * @param       object  $subject The object to observe
	 * @param       array   $config  An array that holds the plugin configuration
	 * @since       1.0
	 */
	function plgSystemmailto2ce( &$subject, $config )
	{
		parent::__construct( $subject, $config );

		// Do some extra initialisation in this constructor if required
	}
	
	/**
	 * Genarate a search pattern based on link and text.
	 *
	 * @param string The target of an e-mail link.
	 * @param string The text enclosed by the link.
	 * @return string A regular expression that matches a link containing the
	 * parameters.
	 */
	function _email_searchPattern ($link, $text) {
		// <a href="mailto:anyLink">anyText</a>
		$pattern = '~(?:<a [\w "\'=\@\.\-]*href\s*=\s*"(mailto:|https?://(?:[a-z0-9][a-z0-9\-]*[a-z0-9]\.)*(?:[a-z0-9]+)(?::\d+)?[a-z0-9;/\?:\@&=+\$,\-_\.!\~*\'\(\)%]+?%3C)'
			. $link . '(%3E)?"([\w "\'=\@\.\-]*))>' . $text . '</a>~i';
	
		return $pattern;
	}
		
	/**
	 * Changes all email links (mailto:) to a link to a Contact Enhanced form.
	 *
	 * @param string The string to be cloaked.
	 * @param string The mode.
	 * replaces addresses with "mailto:" links if nonzero.
	 * @return boolean True on success.
	 */
	function replacer(&$text)
	{
		
		// any@email.address.com
		$searchEmail = '([\w\.\-]+\@(?:[a-z0-9\.\-]+\.)+(?:[a-z0-9\-]{2,4}))';
		// any@email.address.com?subject=anyText
		$searchEmailLink = $searchEmail . '([?&][\x20-\x7f][^"<>]+)';
		// anyText
		$searchText = '((?:[\x20-\x7f]|[\xA1-\xFF]|[\xC2-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF4][\x80-\xBF]{3})[^<>]+)';
	
		//$searchText = '(+)';
		//Any Image link
		$searchImage	=	"(<img[^>]+>)";
	
		/*
		 * Search for derivatives of link code <a href="mailto:email@amail.com"
		 * >email@amail.com</a>
		 */
		$pattern = $this->_email_searchPattern($searchEmail, $searchEmail);
		while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
			$mail = $regs[2][0];
			$mailText = $regs[5][0]; //$mailText = $regs[3][0];
//echo '<pre>'; print_r($regs); exit;
			// Check to see if mail text is different from mail addy
			$replacement = $this->createLink( $mail, $mailText);
	
			// Replace the found address with the link to Contact Enhanced
			$text = substr_replace($text, $replacement, $regs[0][1], strlen($regs[0][0]));
		}
	//echo '<pre>'; print_r($regs); exit;
		/*
		 * Search for derivatives of link code <a href="mailto:email@amail.com">
		 * anytext</a>
		 */
		$pattern = $this->_email_searchPattern($searchEmail, $searchText);
		while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
			
	        $prefix = $regs[1][0];
			$mail = $regs[2][0];
	        $suffix = $regs[3][0];
	        $attribs = $regs[4][0];
			$mailText = $regs[5][0];
	
			$replacement = $this->createLink( $mail, $mailText, $attribs);
	
			// Replace the found address with the link to Contact Enhanced
			$text = substr_replace($text, $replacement, $regs[0][1], strlen($regs[0][0]));
		}
	
		/*
		 * Search for derivatives of link code <a href="mailto:email@amail.com">
		 * <img anything></a>
		 */
		$pattern = $this->_email_searchPattern($searchEmail, $searchImage);
		while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
			$prefix = $regs[1][0];
			$mail = $regs[2][0];
			$suffix = $regs[3][0];
			$attribs = $regs[4][0];
			$mailText = $regs[5][0];
	
			$replacement = $this->createLink( $mail, $mailText, $attribs);
	
			// Replace the found address with the link to Contact Enhanced
			$text = substr_replace($text, $replacement, $regs[0][1], strlen($regs[0][0]));
		}
		
		
	
		/*
		 * Search for derivatives of link code <a href="mailto:email@amail.com?
		 * subject=Text">email@amail.com</a>
		 */
		$pattern = $this->_email_searchPattern($searchEmailLink, $searchEmail);
		while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
			$mail = $regs[2][0] . $regs[3][0];
			$mailText = $regs[6][0];
			
			$queryString = '';
			if(strpos($mail,'?')){
				$queryString = substr($mail, (strpos($mail,'?')+1));
			}
			
			// Needed for handling of Body parameter
			//$mail = str_replace( '&amp;', '&', $mail );
			// This will ignore the subject if it was set in the link
			$mail = substr($mail,0,strpos($mail,'?'));
			// Check to see if mail text is different from mail addy
			$replacement = $this->createLink( $mail, $mailText,'',$queryString);
	
			// Replace the found address with the link to Contact Enhanced
			$text = substr_replace($text, $replacement, $regs[0][1], strlen($regs[0][0]));
		}
	
		/*
		 * Search for derivatives of link code <a href="mailto:email@amail.com?
		 * subject=Text">anytext</a>
		 */
		$pattern = $this->_email_searchPattern($searchEmailLink, $searchText);
		while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
			$mail = $regs[2][0] . $regs[3][0];
			$mailText = $regs[6][0];
			
			$queryString = '';
			if(strpos($mail,'?')){
				$queryString = substr($mail, (strpos($mail,'?')+1));
			}
			
			
			// Needed for handling of Body parameter
			//$mail = str_replace( '&amp;', '&', $mail );
			// This will ignore the subject if it was set in the link
			$mail = substr($mail,0,strpos($mail,'?'));
			
			$replacement = $this->createLink( $mail, $mailText,'',$queryString);
	
			// Replace the found address with the link to Contact Enhanced
			$text = substr_replace($text, $replacement, $regs[0][1], strlen($regs[0][0]));
		}
		
		
			// Search for plain text email@amail.com
			$pattern = '~' . $searchEmail . '([^a-z0-9]|$)~i';
			while (preg_match($pattern, $text, $regs, PREG_OFFSET_CAPTURE)) {
				$mail = $regs[1][0];
				if($this->params->get('parse-plain-text-emails',0)){
					$replacement = $this->createLink( $mail);
				}else{
					$replacement = $this->_convertEncoding($mail);
				}
				// Replace the found address with the link to Contact Enhanced
				$text = substr_replace($text, $replacement, $regs[1][1], strlen($mail));
			}
		
		return true;
	}
	
	function createLink($mail,$mailText=null, $attribs='',$queryString='') {
		
		$doc	= &JFactory::getDocument();
		require_once (JPATH_ROOT.DS.'components'.DS.'com_contact_enhanced'.DS.'helper'.DS.'helper.php');
		
			
		$link	= ('index.php?option=com_contact_enhanced&view=contact&id='
						.$this->params->get('contactid',1)
						.'&content_title='.ceHelper::encode($doc->getTitle())
						.'&content_url='.ceHelper::encode(ceHelper::getCurrentURL())
						.'&recipient='.ceHelper::encode($mail)
						.($this->params->get('modal',1) ? '&tmpl=component' : '') 
						.($this->params->get('template') ? '&template='.$this->params->get('template') : '')
						.'&amp;'.$queryString 
					);
		
		$mailText = ($mailText ? $mailText : $mail);
		if(strpos($mailText,'@')){
			$mailText	= $this->_convertEncoding($mailText);
		}
		$link	= '<a	href="'.JURI::root().$link.'" 
						class="modal mailto2ce" 
						'.$attribs.'
						rel="{handler: \'iframe\', size: {x:'.$this->params->get('window-size-width',800).', y:'.$this->params->get('window-size-height',480).'}}"
						>'
						.$mailText
						//.($mailText ? $mailText : $mail)
					.'</a>';
		return $link;
	}

	function onAfterRender()
	{
		$app = JFactory::getApplication();
		
		$body = JResponse::getBody();
		
		// Simple performance check to determine whether bot should process further.
		if (stripos($body, '@') === false 
			OR $app->isAdmin()
			OR JRequest::getVar('option') == 'com_contact_enhanced'
			OR JRequest::getVar('option') == 'com_irecommend' 
			) {
				
			if($app->isAdmin()){
				$emailcloakPlugin = & JPluginHelper::getPlugin('content', 'emailcloak');
				//echo '<pre>'; print_r($emailcloakPlugin); exit;
				if(is_object($emailcloakPlugin)){
					// disables emailcloak plugin
					$db	= JFactory::getDBO();
					$db->setQuery('UPDATE #__plugins SET published = 0 WHERE element = '.$db->Quote('emailcloak'));
					$db->query();
				}
			}
			return true;
		}
		
		$this->replacer($body);
		JResponse::setBody($body);
	}
	
	function onAfterInitialise(){
		$app = JFactory::getApplication();
		if(!$app->isAdmin()){
			JHTML::_('behavior.mootools');
			JHTML::_('behavior.modal', 'a.modal');
		}
	}
	
	function _convertEncoding( $text )
	{
		// replace vowels with character encoding
		$text 	= str_replace( 'a', '&#97;', $text );
		$text 	= str_replace( 'e', '&#101;', $text );
		$text 	= str_replace( 'i', '&#105;', $text );
		$text 	= str_replace( 'o', '&#111;', $text );
		$text	= str_replace( 'u', '&#117;', $text );
		
		// replace @ with character encoding
		$text	= str_replace( '@', '&#64;', $text );
		

		return $text;
	}
}

